==> fraiseql-1.1.4.tar.gz: fraiseql-1.1.4/tests/integration/graphql/test_graphql_query_execution_complete.py <==
"""Integration tests for direct path GraphQL execution.

These tests validate the direct path: GraphQL → SQL → Rust → HTTP.
This bypasses GraphQL resolvers entirely for maximum performance.

Pipeline: Query parsing → SQL generation → JSONB retrieval →
          Rust transformation (camelCase + field projection + __typename) → HTTP

Status: ✅ PASSING - Direct path implemented and working!
"""

import pytest
from httpx import AsyncClient, ASGITransport

from fraiseql import type as fraiseql_type, query
from fraiseql.fastapi import create_fraiseql_app
from fraiseql.sql import create_graphql_where_input


@pytest.mark.asyncio
async def test_graphql_simple_query_returns_data(create_fraiseql_app_with_db, db_connection):
    """Test that simple GraphQL query returns data via direct path.

    ✅ Tests: GraphQL → SQL → Rust → HTTP for single object query.
    """
    # Setup test data
    await db_connection.execute("""
        DROP TABLE IF EXISTS tv_user CASCADE;
        DROP VIEW IF EXISTS v_user CASCADE;

        CREATE TABLE tv_user (
            id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
            data JSONB NOT NULL
        );

        INSERT INTO tv_user (id, data) VALUES
            ('11111111-1111-1111-1111-111111111111', '{"id": "11111111-1111-1111-1111-111111111111", "first_name": "John", "last_name": "Doe", "email": "john@example.com"}');

        CREATE VIEW v_user AS
        SELECT id, data FROM tv_user;
    """)
    await db_connection.commit()

    @fraiseql_type(sql_source="v_user", jsonb_column="data")
    class User:
        id: str
        first_name: str
        email: str

    @query
    async def user(info, id: str) -> User | None:
        db = info.context["db"]
        return await db.find_one("v_user", info=info, id=id)

    app = create_fraiseql_app_with_db(
        types=[User],
        queries=[user]
    )

    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        response = await client.post(
            "/graphql",
            json={
                "query": 'query { user(id: "11111111-1111-1111-1111-111111111111") { id firstName email } }'
            }
        )

    data = response.json()

    # Verify direct path success
    assert "errors" not in data, f"Query failed with errors: {data.get('errors')}"
    assert data["data"]["user"]["id"] == "11111111-1111-1111-1111-111111111111"
    assert data["data"]["user"]["firstName"] == "John"
    assert data["data"]["user"]["email"] == "john@example.com"


@pytest.mark.asyncio
async def test_graphql_list_query_returns_array(create_fraiseql_app_with_db, db_connection):
    """Test that list queries return arrays via direct path.

    ✅ Tests: GraphQL → SQL → Rust → HTTP for list queries.
    """
    # Setup test data
    await db_connection.execute("""
        DROP TABLE IF EXISTS tv_user CASCADE;
        DROP VIEW IF EXISTS v_user CASCADE;

        CREATE TABLE tv_user (
            id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
            data JSONB NOT NULL
        );

        INSERT INTO tv_user (id, data) VALUES
            ('11111111-1111-1111-1111-111111111111', '{"id": "11111111-1111-1111-1111-111111111111", "first_name": "John", "last_name": "Doe"}'),
            ('22222222-2222-2222-2222-222222222222', '{"id": "22222222-2222-2222-2222-222222222222", "first_name": "Jane", "last_name": "Smith"}'),
            ('33333333-3333-3333-3333-333333333333', '{"id": "33333333-3333-3333-3333-333333333333", "first_name": "Bob", "last_name": "Johnson"}');

        CREATE VIEW v_user AS
        SELECT id, data FROM tv_user;
    """)
    await db_connection.commit()

    @fraiseql_type(sql_source="v_user", jsonb_column="data")
    class User:
        id: str
        first_name: str

    @query
    async def users(info, limit: int = 10) -> list[User]:
        db = info.context["db"]
        return await db.find("v_user", info=info, limit=limit)

    app = create_fraiseql_app_with_db(
        types=[User],
        queries=[users]
    )

    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        response = await client.post(
            "/graphql",
            json={"query": "query { users(limit: 5) { id firstName } }"}
        )

    data = response.json()

    assert "errors" not in data, f"Query failed: {data.get('errors')}"
    assert isinstance(data["data"]["users"], list)
    assert len(data["data"]["users"]) == 3
    assert all("id" in user and "firstName" in user for user in data["data"]["users"])


@pytest.mark.asyncio
async def test_graphql_field_selection(create_fraiseql_app_with_db, db_connection):
    """Test that Rust field projection works correctly.

    ✅ Tests: Rust filters fields to only those requested in GraphQL query.
    """
    # Setup test data
    await db_connection.execute("""
        DROP TABLE IF EXISTS tv_user CASCADE;
        DROP VIEW IF EXISTS v_user CASCADE;

        CREATE TABLE tv_user (
            id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
            data JSONB NOT NULL
        );

        INSERT INTO tv_user (id, data) VALUES
            ('11111111-1111-1111-1111-111111111111', '{"id": "11111111-1111-1111-1111-111111111111", "first_name": "John", "last_name": "Doe", "email": "john@example.com"}');

        CREATE VIEW v_user AS
        SELECT id, data FROM tv_user;
    """)
    await db_connection.commit()

    @fraiseql_type(sql_source="v_user", jsonb_column="data")
    class User:
        id: str
        first_name: str
        last_name: str
        email: str

    @query
    async def user(info, id: str) -> User | None:
        db = info.context["db"]
        return await db.find_one("v_user", info=info, id=id)

    app = create_fraiseql_app_with_db(
        types=[User],
        queries=[user]
    )

    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        # Request only specific fields
        response = await client.post(
            "/graphql",
            json={
                "query": 'query { user(id: "11111111-1111-1111-1111-111111111111") { id firstName } }'
            }
        )

    data = response.json()

    assert "errors" not in data, f"Query failed: {data.get('errors')}"
    user_data = data["data"]["user"]

    # Should have requested fields
    assert "id" in user_data
    assert "firstName" in user_data

    # Should NOT have non-requested fields (Rust field projection)
    assert "email" not in user_data
    assert "lastName" not in user_data


@pytest.mark.asyncio
async def test_graphql_with_where_filter(create_fraiseql_app_with_db, db_connection):
    """Test GraphQL queries with WHERE filters via direct path.

    ✅ Tests: WHERE filters work with dict arguments in direct path.
    """
    # Setup test data
    await db_connection.execute("""
        DROP TABLE IF EXISTS tv_user CASCADE;
        DROP VIEW IF EXISTS v_user CASCADE;

        CREATE TABLE tv_user (
            id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
            data JSONB NOT NULL
        );

        INSERT INTO tv_user (id, data) VALUES
            ('11111111-1111-1111-1111-111111111111', '{"id": "11111111-1111-1111-1111-111111111111", "first_name": "John", "active": true}'),
            ('22222222-2222-2222-2222-222222222222', '{"id": "22222222-2222-2222-2222-222222222222", "first_name": "Jane", "active": false}'),
            ('33333333-3333-3333-3333-333333333333', '{"id": "33333333-3333-3333-3333-333333333333", "first_name": "Bob", "active": true}');

        CREATE VIEW v_user AS
        SELECT id, data FROM tv_user;
    """)
    await db_connection.commit()

    @fraiseql_type(sql_source="v_user", jsonb_column="data")
    class User:
        id: str
        first_name: str
        active: bool

    # Generate Where input type
    UserWhereInput = create_graphql_where_input(User)

    @query
    async def users(info, where: UserWhereInput | None = None) -> list[User]:
        db = info.context["db"]
        return await db.find("v_user", info=info, where=where)

    app = create_fraiseql_app_with_db(
        types=[User, UserWhereInput],
        queries=[users]
    )

    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        response = await client.post(
            "/graphql",
            json={
                "query": '''
                    query {
                        users(where: {active: {eq: true}}) {
                            id
                            firstName
                            active
                        }
                    }
                '''
            }
        )

    data = response.json()

    assert "errors" not in data, f"Query failed: {data.get('errors')}"
    users_data = data["data"]["users"]
    assert len(users_data) == 2  # John and Bob
    assert all(user["active"] for user in users_data)

==> fraiseql-1.3.1.tar.gz: fraiseql-1.3.1/tests/integration/graphql/test_graphql_query_execution_complete.py <==
"""Integration tests for direct path GraphQL execution.

These tests validate the direct path: GraphQL → SQL → Rust → HTTP.
This bypasses GraphQL resolvers entirely for maximum performance.

Pipeline: Query parsing → SQL generation → JSONB retrieval →
          Rust transformation (camelCase + field projection + __typename) → HTTP

Status: ✅ PASSING - Direct path implemented and working!
"""

import pytest
from httpx import AsyncClient, ASGITransport

from fraiseql import type as fraiseql_type, query
from fraiseql.fastapi import create_fraiseql_app
from fraiseql.sql import create_graphql_where_input


@pytest.mark.asyncio
async def test_graphql_simple_query_returns_data(create_fraiseql_app_with_db, db_connection):
    """Test that simple GraphQL query returns data via direct path.

    ✅ Tests: GraphQL → SQL → Rust → HTTP for single object query.
    """
    # Setup test data
    await db_connection.execute("""
        DROP TABLE IF EXISTS tv_user CASCADE;
        DROP VIEW IF EXISTS v_user CASCADE;

        CREATE TABLE tv_user (
            id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
            data JSONB NOT NULL
        );

        INSERT INTO tv_user (id, data) VALUES
            ('11111111-1111-1111-1111-111111111111', '{"id": "11111111-1111-1111-1111-111111111111", "first_name": "John", "last_name": "Doe", "email": "john@example.com"}');

        CREATE VIEW v_user AS
        SELECT id, data FROM tv_user;
    """)
    await db_connection.commit()

    @fraiseql_type(sql_source="v_user", jsonb_column="data")
    class User:
        id: str
        first_name: str
        email: str

    @query
    async def user(info, id: str) -> User | None:
        db = info.context["db"]
        return await db.find_one("v_user", info=info, id=id)

    app = create_fraiseql_app_with_db(
        types=[User],
        queries=[user]
    )

    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        response = await client.post(
            "/graphql",
            json={
                "query": 'query { user(id: "11111111-1111-1111-1111-111111111111") { id firstName email } }'
            }
        )

    data = response.json()

    # Verify direct path success
    assert "errors" not in data, f"Query failed with errors: {data.get('errors')}"
    assert data["data"]["user"]["id"] == "11111111-1111-1111-1111-111111111111"
    assert data["data"]["user"]["firstName"] == "John"
    assert data["data"]["user"]["email"] == "john@example.com"


@pytest.mark.asyncio
async def test_graphql_list_query_returns_array(create_fraiseql_app_with_db, db_connection):
    """Test that list queries return arrays via direct path.

    ✅ Tests: GraphQL → SQL → Rust → HTTP for list queries.
    """
    # Setup test data
    await db_connection.execute("""
        DROP TABLE IF EXISTS tv_user CASCADE;
        DROP VIEW IF EXISTS v_user CASCADE;

        CREATE TABLE tv_user (
            id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
            data JSONB NOT NULL
        );

        INSERT INTO tv_user (id, data) VALUES
            ('11111111-1111-1111-1111-111111111111', '{"id": "11111111-1111-1111-1111-111111111111", "first_name": "John", "last_name": "Doe"}'),
            ('22222222-2222-2222-2222-222222222222', '{"id": "22222222-2222-2222-2222-222222222222", "first_name": "Jane", "last_name": "Smith"}'),
            ('33333333-3333-3333-3333-333333333333', '{"id": "33333333-3333-3333-3333-333333333333", "first_name": "Bob", "last_name": "Johnson"}');

        CREATE VIEW v_user AS
        SELECT id, data FROM tv_user;
    """)
    await db_connection.commit()

    @fraiseql_type(sql_source="v_user", jsonb_column="data")
    class User:
        id: str
        first_name: str

    @query
    async def users(info, limit: int = 10) -> list[User]:
        db = info.context["db"]
        return await db.find("v_user", info=info, limit=limit)

    app = create_fraiseql_app_with_db(
        types=[User],
        queries=[users]
    )

    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        response = await client.post(
            "/graphql",
            json={"query": "query { users(limit: 5) { id firstName } }"}
        )

    data = response.json()

    assert "errors" not in data, f"Query failed: {data.get('errors')}"
    assert isinstance(data["data"]["users"], list)
    assert len(data["data"]["users"]) == 3
    assert all("id" in user and "firstName" in user for user in data["data"]["users"])


@pytest.mark.skip(reason="Schema registry singleton - only one initialization per process. Test passes individually. Run with: pytest tests/integration/graphql/test_graphql_query_execution_complete.py::test_graphql_field_selection -v")
@pytest.mark.asyncio
async def test_graphql_field_selection(create_fraiseql_app_with_db, db_connection):
    """Test that Rust field projection works correctly.

    ✅ Tests: Rust filters fields to only those requested in GraphQL query.
    """
    # Setup test data
    await db_connection.execute("""
        DROP TABLE IF EXISTS tv_user CASCADE;
        DROP VIEW IF EXISTS v_user CASCADE;

        CREATE TABLE tv_user (
            id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
            data JSONB NOT NULL
        );

        INSERT INTO tv_user (id, data) VALUES
            ('11111111-1111-1111-1111-111111111111', '{"id": "11111111-1111-1111-1111-111111111111", "first_name": "John", "last_name": "Doe", "email": "john@example.com"}');

        CREATE VIEW v_user AS
        SELECT id, data FROM tv_user;
    """)
    await db_connection.commit()

    @fraiseql_type(sql_source="v_user", jsonb_column="data")
    class User:
        id: str
        first_name: str
        last_name: str
        email: str

    @query
    async def user(info, id: str) -> User | None:
        db = info.context["db"]
        return await db.find_one("v_user", info=info, id=id)

    app = create_fraiseql_app_with_db(
        types=[User],
        queries=[user]
    )

    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        # Request only specific fields
        response = await client.post(
            "/graphql",
            json={
                "query": 'query { user(id: "11111111-1111-1111-1111-111111111111") { id firstName } }'
            }
        )

    data = response.json()

    assert "errors" not in data, f"Query failed: {data.get('errors')}"
    user_data = data["data"]["user"]

    # Should have requested fields
    assert "id" in user_data
    assert "firstName" in user_data

    # Should NOT have non-requested fields (Rust field projection)
    assert "email" not in user_data
    assert "lastName" not in user_data


@pytest.mark.asyncio
async def test_graphql_with_where_filter(create_fraiseql_app_with_db, db_connection):
    """Test GraphQL queries with WHERE filters via direct path.

    ✅ Tests: WHERE filters work with dict arguments in direct path.
    """
    # Setup test data
    await db_connection.execute("""
        DROP TABLE IF EXISTS tv_user CASCADE;
        DROP VIEW IF EXISTS v_user CASCADE;

        CREATE TABLE tv_user (
            id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
            data JSONB NOT NULL
        );

        INSERT INTO tv_user (id, data) VALUES
            ('11111111-1111-1111-1111-111111111111', '{"id": "11111111-1111-1111-1111-111111111111", "first_name": "John", "active": true}'),
            ('22222222-2222-2222-2222-222222222222', '{"id": "22222222-2222-2222-2222-222222222222", "first_name": "Jane", "active": false}'),
            ('33333333-3333-3333-3333-333333333333', '{"id": "33333333-3333-3333-3333-333333333333", "first_name": "Bob", "active": true}');

        CREATE VIEW v_user AS
        SELECT id, data FROM tv_user;
    """)
    await db_connection.commit()

    @fraiseql_type(sql_source="v_user", jsonb_column="data")
    class User:
        id: str
        first_name: str
        active: bool

    # Generate Where input type
    UserWhereInput = create_graphql_where_input(User)

    @query
    async def users(info, where: UserWhereInput | None = None) -> list[User]:
        db = info.context["db"]
        return await db.find("v_user", info=info, where=where)

    app = create_fraiseql_app_with_db(
        types=[User, UserWhereInput],
        queries=[users]
    )

    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        response = await client.post(
            "/graphql",
            json={
                "query": '''
                    query {
                        users(where: {active: {eq: true}}) {
                            id
                            firstName
                            active
                        }
                    }
                '''
            }
        )

    data = response.json()

    assert "errors" not in data, f"Query failed: {data.get('errors')}"
    users_data = data["data"]["users"]
    assert len(users_data) == 2  # John and Bob
    assert all(user["active"] for user in users_data)
